.initApplera <- function(where){
require("Biobase", quietly = FALSE) || stop("cannot load applera without Biobase")
require("affy", quietly = FALSE) || stop("cannot load applera without affy")
require("genefilter", quietly = FALSE) || stop("cannot load applera without genefilter")
require("limma", quietly = FALSE)  || stop("cannot load applera without limma")
#require("hexbin") || stop("cannot load applera without hexbin")
require("geneplotter", quietly = FALSE) || stop("cannot load applera without geneplotter")
require("RColorBrewer", quietly = FALSE) || stop("cannot load applera without RColorBrewer")
require("prada", quietly = FALSE) || stop("cannot load applera without prada")
require("combinat", quietly = FALSE) || stop("cannot load applera without combinat")

###############################################################################################
#applera class definitions                                                                    #
###############################################################################################
#definining the class containing all applera informations
#each applera info are defined by an exprSet object
setClass("applera",

         representation(Organism="character", Geneid="character", Signal="exprSet", Sdev="exprSet", Cv="exprSet", 
                       Sn="exprSet", Flags="exprSet", Ctrl="list"), contains="exprSet", where=where
)

#setClass("applera",
#
#         representation(Organism="character", Geneid="character", Signal="exprSet", Sdev="exprSet", Cv="exprSet", 
#                        Sn="exprSet", Flags="exprSet", Ctrl="list"), contains="exprSet"
#)



#these methods are used to access to the single slots of the applera class
#Organism slot
#used to define the size of the genes in the arrays without the controls
if(is.null(getGeneric("organism"))) setGeneric("organism", function(object) standardGeneric("organism"))

#Organism is a method to access to the corresponding applera slot
setMethod("organism", signature(object="applera"),
          function(object) {object@Organism}
)

#Primary geneId slot
#used to define the size of the genes in the arrays without the controls
if(is.null(getGeneric("geneid")))setGeneric("geneid", function(object) standardGeneric("geneid"))
setMethod("geneid", signature(object="applera"), function(object) { 
	           primaryId <- as.vector(object@Geneid)
	           names(primaryId) <- geneNames(object@Signal)
		   return(primaryId)
	  }
)

#Signal slot
if(is.null(getGeneric("signal"))) setGeneric("signal", function(object) standardGeneric("signal"))
setMethod("signal", signature(object="applera"),
          function(object) {object@Signal}
)
#Sdev slot
#Estimated measurement uncertainty of Normalized Signal
#not log2
if(is.null(getGeneric("sdev"))) setGeneric("sdev", function(object) standardGeneric("sdev"))
setMethod("sdev", signature(object="applera"),
          function(object) {object@Sdev}
)

#Cv slot
#Fractional uncetainty in the signal intecates the predicted spread of the feature signal
#not log2
if(is.null(getGeneric("cvAp"))) setGeneric("cvAp", function(object) standardGeneric("cvAp"))
setMethod("cvAp", signature(object="applera"),
          function(object) {object@Cv}
)

#Sn slot
#Ratio of Signal/SDEV. Expresses the confidence of feature detectability.
#not log2
if(is.null(getGeneric("sn"))) setGeneric("sn", function(object) standardGeneric("sn"))
setMethod("sn", signature(object="applera"),
          function(object) {object@Sn}
)

#Flags slot
#Numeric code that identifies conditions for each feature.
#not log2
if(is.null(getGeneric("flags"))) setGeneric("flags", function(object) standardGeneric("flags"))
setMethod("flags", signature(object="applera"),
          function(object) {object@Flags}
)

#Ctrl slot contains only the signals associated to ctrls in applera output files
#Numeric code that identifies conditions for each feature.
#not log2
if(is.null(getGeneric("ctrl"))) setGeneric("ctrl", function(object) standardGeneric("ctrl"))
setMethod("ctrl", signature(object="applera"),
          function(object) {object@Ctrl}
)



#this method give the characteristics of the applera  instance
#OK
setMethod("show",signature(object="applera"),
          function(object){
               cat("This is an instance of applera-class","\n\t")
               cat("Slots can be accessed using:\n\t") 
               cat("signal, sdev, cv, sn, flags methods","\n\t")
               cat("Each slot is an exprSet object made of","\n\t")
               ngenes <- dim(object@Signal@exprs)[1]
               nctrl <- dim(object@Ctrl$"signal")[1]
               nsamples <- dim(object@Signal@exprs)[2]
               cat(ngenes, " genes\n\t", sep="")
               cat(nctrl, " controls\n\t", sep="")
               cat(nsamples, " samples\n")             
}) 

#hist applera method
#this method needs the applera instance and the range of columns to be plotted e.g. c(1:3)
#the method discard all the applera controls 
#
setMethod("hist",signature(x="applera"), function(x, i){
                 par(mfrow=c(2,3))
                 plotDensity(exprs(signal(x)[,i]), xlab="log2(signal)")
                 plotDensity(log2(exprs(sdev(x)[,i])), xlab="log2(sdev)")
                 plotDensity(log2(exprs(cvAp(x)[,i])), xlab="log2(cvAp)")
                 plotDensity(exprs(sn(x)[,i]), xlab="sn")
                 plotDensity(exprs(flags(x)[,i]), xlab="flags")
})

#this method produces a box plot for all the 5 slots of an applera object
#
setMethod("boxplot",signature(x="applera"), function(x){
                     par(mfrow=c(2,3))
                     #signal boxplot
                     tmp<-exprs(signal(x))
                     expnames<-dimnames(tmp)[[2]]
                     boxplot(as.data.frame(tmp))
                     title("log2(signal)")

                     #sdev boxplot
                     tmp<-log2(exprs(sdev(x)))
                     expnames<-dimnames(tmp)[[2]]
                     boxplot(as.data.frame(tmp))
                     title("log2(sdev)")

                     #cv boxplot
                     tmp<-log2(exprs(cvAp(x)))
                     expnames<-dimnames(tmp)[[2]]
                     boxplot(as.data.frame(tmp))
                     title("log2(cvAp)")

                     #sn boxplot
                     tmp<-exprs(sn(x))
                     expnames<-dimnames(tmp)[[2]]
                     boxplot(as.data.frame(tmp))
                     title("sn")

                     #Flags boxplot
                     tmp<-exprs(flags(x))
                     expnames<-dimnames(tmp)[[2]]
                     boxplot(as.data.frame(tmp))
                     title("flags")
})



#subsetting applera objects
#This method needs  an applera object (apObj) and a vector (genes) with the requested subset of applera ids
setGeneric("subGenes", function(apObj, genes) standardGeneric("subGenes"))
setMethod("subGenes",signature(apObj="applera"), function(apObj, genes){
                 signal.tmp<-signal(apObj)[geneNames(signal(apObj))%in%genes,]
                 sdev.tmp<-sdev(apObj)[geneNames(sdev(apObj))%in%genes,]
                 cv.tmp<-cvAp(apObj)[geneNames(cvAp(apObj))%in%genes,]
                 sn.tmp<-sn(apObj)[geneNames(sn(apObj))%in%genes,]
                 flags.tmp<-flags(apObj)[geneNames(flags(apObj))%in%genes,]
                 myclass<-new("applera")
                 myclass@Organism<-apObj@Organism
                 myclass@Signal<-signal.tmp
                 myclass@Sdev<-sdev.tmp
                 myclass@Cv<-cv.tmp
                 myclass@Sn<-sn.tmp
                 myclass@Flags<-flags.tmp
                 myclass@Ctrl<-apObj@Ctrl
                 return(myclass)
})

#subsetting by samples
#the function needs apObj an applera object and a vector with the sampleNames subset 
setGeneric("subExp", function(apObj, samples) standardGeneric("subExp"))
setMethod("subExp",signature(apObj="applera"), function(apObj, samples){
                 signal.tmp<-signal(apObj)[,sampleNames(signal(apObj))%in%samples]
                 sdev.tmp<-sdev(apObj)[,sampleNames(sdev(apObj))%in%samples]
                 cv.tmp<-cvAp(apObj)[,sampleNames(cvAp(apObj))%in%samples]
                 sn.tmp<-sn(apObj)[,sampleNames(sn(apObj))%in%samples]
                 flags.tmp<-flags(apObj)[,sampleNames(flags(apObj))%in%samples]
                 myclass<-new("applera")
                 myclass@Organism<-apObj@Organism
                 myclass@Signal<-signal.tmp
                 myclass@Sdev<-sdev.tmp
                 myclass@Cv<-cv.tmp
                 myclass@Sn<-sn.tmp
                 myclass@Flags<-flags.tmp
                 myclass@Ctrl<-apObj@Ctrl
                 return(myclass)
})

#normalizing sample
#the function needs apObj, an applera object, and it will normalize the Signal 
#using the "normalize.loess" method from the affy library
#require(affy)
setGeneric("normLoess", function(apObj) standardGeneric("normLoess"))
setMethod("normLoess",signature(apObj="applera"), function(apObj){
                 tmp<-exprs(signal(apObj))#the matrix to be normalized
                 signal.tmp<-normalize.loess(tmp)
                 eset <- new("exprSet", exprs=signal.tmp, phenoData=phenoData(signal(apObj)))
                 myclass<-new("applera")
                 myclass@Organism<-apObj@Organism
                 myclass@Geneid<-apObj@Geneid
                 myclass@Signal <- eset
                 myclass@Sdev <- apObj@Sdev
                 myclass@Cv <- apObj@Cv
                 myclass@Sn <- apObj@Sn
                 myclass@Flags <- apObj@Flags
                 myclass@Ctrl <- apObj@Ctrl
                 return(myclass)
})
#the function needs apObj, an applera object, and it will normalize the Signal 
#using the "normalize.quantiles" method from the affy library
#require(affy)
setGeneric("normQuantile", function(apObj) standardGeneric("normQuantile"))
setMethod("normQuantile",signature(apObj="applera"), function(apObj){
                 tmp<-exprs(signal(apObj))#the matrix to be normalized
                 signal.tmp<-normalize.quantiles(tmp)
                 eset <- new("exprSet", exprs=signal.tmp, phenoData=phenoData(signal(apObj)))
                 myclass<-new("applera")
                 myclass@Organism<-apObj@Organism
                 myclass@Geneid<-apObj@Geneid
                 myclass@Signal <- eset
                 myclass@Sdev <- apObj@Sdev
                 myclass@Cv <- apObj@Cv
                 myclass@Sn <- apObj@Sn
                 myclass@Flags <- apObj@Flags
                 myclass@Ctrl <- apObj@Ctrl
                 return(myclass)
})


#This method is used to filter applera data on the bases of S/N data
#The method needs an object of applera class, a p-value ranging 0 - 1 indicating the minimum % of array
#with a specific condition and A indicates the condition of the filtering default p=1 and A=3
#A=3 indicates 3-SDEV confidene (99.9%) that the signal measurement is real
#or a flag 
setGeneric("apFilter", function(apObj, p, A, type) standardGeneric("apFilter"))
setMethod("apFilter",signature(apObj="applera"), function(apObj, p, A, type){
                      if(type=="sn"){
                          if(p>1) {cat("Error:\n\tp-value cannot be greater than 1\n"); return(NULL)}
                          if(p<0) {cat("Error:\n\tp-value cannot be negative\n"); return(NULL)}
                          if(A>max(exprs(sn(apObj)))) {cat("Error:\n\tA cannot be greater than ", 
                                                          max(exprs(sn(apObj))), "\n") 
                                                       return(NULL)
                                                      }
                          fSn <- pOverA(p=p, A=A)
                          filterPar <- paste(c("p=", p, ", A=", A), collapse="")
                          ffun <- filterfun(fSn)
                          which <- genefilter(exprs(sn(apObj)), ffun)
                          par(mfrow=c(2,2))
                          my.title <- paste( "Filtering params:", filterPar, sep="\n")
                          plot(density(exprs(sn(apObj))), main=my.title,xlab="S/N")
                          my.title <- paste( "All signal\n genes=",  dim(exprs(signal(apObj)))[1], sep=" ")
                          plot(density(exprs(signal(apObj))), main=my.title, xlab="signal")
                          my.title <- paste( "Filtering params:", filterPar, sep="\n")
                          plot(density(exprs(sn(apObj))[which,]), main=my.title, xlab="S/N")
                          my.title <- paste( "Filtered signal\n genes left=", sum(which), sep=" ")
                          plot(density(exprs(signal(apObj))[which,]), main=my.title, xlab="signal")
                          filtered.geneN<-geneNames(signal(apObj)[which,])
                          result <- subGenes(apObj, filtered.geneN)
                          return(result)
                     }

                     if(type=="flags"){
                          if(p>1) {cat("Error:\n\tp-value cannot be greater than 1\n"); return(NULL)}
                          if(p<0) {cat("Error:\n\tp-value cannot be negative\n"); return(NULL)}
                          if(A>max(exprs(flags(apObj)))) {cat("Error:\n\tA cannot be greater than ", 
                                                             max(exprs(flags(apObj))), "\n") 
                                                          return(NULL)
                                                         }
                          fSn <- pOverA(p=p, A=A)
                          filterPar <- paste(c("p=", p, ", A=", A), collapse="")
                          ffun <- filterfun(fSn)
                          which <- genefilter(exprs(flags(apObj)), ffun)
			  par(mfrow=c(2,2))
                          my.title <- paste( "Filtering params:", filterPar, sep="\n")
                          plot(density(exprs(flags(apObj))), main=my.title,xlab="flags")
                          my.title <- paste( "All signal\n genes=",  dim(exprs(signal(apObj)))[1], sep=" ")
                          plot(density(exprs(signal(apObj))), main=my.title, xlab="signal")
#                          filtered.geneN <- geneNames(signal(apObj)[which,])
                          filtered.tmp<-geneNames(signal(apObj)[which,])
			  filtered.geneN <- setdiff(geneNames(signal(apObj)), filtered.tmp)
                          result <- subGenes(apObj, filtered.geneN)
                          my.title <- paste( "Filtering params:", filterPar, sep="\n")
                          plot(density(exprs(flags(result))), main=my.title, xlab="flags")
                          my.title <- paste( "Filtered signal\n genes left=", length(filtered.geneN), sep=" ")
                          plot(density(exprs(signal(result))), main=my.title, xlab="signal")
			  return(result)
                     }

})


#This method is used to filter applera data on the bases of IQR range
setGeneric("iqrFilter", function(apObj, threshold) standardGeneric("iqrFilter"))
setMethod("iqrFilter",signature(apObj="applera"), function(apObj, threshold){
                          iqr <- function(x)(IQR(x)>threshold)
                          filterPar <- paste(c("IQR> ", threshold), collapse="")
                          ffun <- filterfun(iqr)
                          which <- genefilter(exprs(signal(apObj)), ffun)
                          par(mfrow=c(1,2))
                          my.title <- paste( "All signal\n genes=",  dim(exprs(signal(apObj)))[1], sep=" ")
                          plot(density(exprs(signal(apObj))), main=my.title, xlab="signal")
                          my.title <- paste( "Filtered signal\n genes left=", sum(which), sep=" ")
                          plot(density(exprs(signal(apObj))[which,]), main=my.title, xlab="signal")
                          filtered.geneN<-geneNames(signal(apObj)[which,])
                          result <- subGenes(apObj, filtered.geneN)
                          return(result)

})



#the function is used to generate an mva.pairs plot
#I have to understand how change the plotting paramenters and pass them to the class 
#require(affy)
setGeneric("mvaAp", function(apObj) standardGeneric("mvaAp"))
setMethod("mvaAp",signature(apObj="applera"), function(apObj){
                 tmp<-exprs(signal(apObj))#the matrix to be normalized
                 mva.pairs(tmp, digits=2, cex=1)
})
}

